@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2016  B.A.T.M.A.N. contributors:
+/* Copyright (C) 2007-2017  B.A.T.M.A.N. contributors:
  *
  * Marek Lindner, Simon Wunderlich
  *
@@ -130,31 +130,10 @@ static int __batadv_interface_kill_vid(struct net_device *dev, __be16 proto,\
 
 #endif /* < KERNEL_VERSION(3, 10, 0) */
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 15, 0)
-
-/* the expected behaviour of this function is to return 0 on success, therefore
- * it is possible to define it as 1 so that batman-adv thinks like something
- * went wrong. It will then decide what to do.
- */
-#define cfg80211_get_station(_a, _b, _c) (1)
-/* the following define substitute the expected_throughput field with a random
- * one existing in the station_info struct. It can be random because due to the
- * define above it will never be used. We need it only to make the code compile
- */
-#define expected_throughput filled
-
-#ifdef CONFIG_BATMAN_ADV_BATMAN_V
-
-#warning cfg80211 based throughput metric is only supported with Linux 3.15+
-
-#endif
-
-#endif /* < KERNEL_VERSION(3, 15, 0) */
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(4, 0, 0)
 
-/* WARNING for batadv_getlink_net */
-#define get_link_net get_xstats_size || 1 || netdev->rtnl_link_ops->get_xstats_size
+/* wild hack for batadv_getlink_net only */
+#define get_link_net get_xstats_size || 1 ? fallback_net : (struct net*)netdev->rtnl_link_ops->get_xstats_size
 
 #endif /* < KERNEL_VERSION(4, 0, 0) */
 
@@ -164,4 +143,13 @@ static int __batadv_interface_kill_vid(struct net_device *dev, __be16 proto,\
 
 #endif /* < KERNEL_VERSION(4, 3, 0) */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 6, 0)
+
+/* workaround for current issues with Debian's make-kpkg */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 5, 0)
+#include <uapi/linux/pkt_cls.h>
+#endif
+
+#endif /* < KERNEL_VERSION(4, 6, 0) */
+
 #endif /* _NET_BATMAN_ADV_COMPAT_H_ */
